from byu_awslogin.assume_role import ask_which_role_to_assume, assume_role

def test_ask_which_role_to_assume_single_role():
    account_names = {
        '333333333333': 'my-dev-account',
        '222222222222': 'my-prod-account'
    }
    principal_roles = [
        ['arn:aws:iam::333333333333:saml-provider/ADFS', 'arn:aws:iam::333333333333:role/ReadOnly'],
        ['arn:aws:iam::333333333333:saml-provider/ADFS', 'arn:aws:iam::333333333333:role/Admin'],
        ['arn:aws:iam::222222222222:saml-provider/ADFS', 'arn:aws:iam::222222222222:role/ReadOnly']
    ]
    account_name = 'my-dev-account'
    role_name = "ReadOnly"

    account_roles = ask_which_role_to_assume(account_names, principal_roles, account_name, role_name)
    assert len(account_roles) == 1
    assert account_roles[0].account_name == "my-dev-account"
    assert account_roles[0].role_name == "ReadOnly"
    assert account_roles[0].principal_arn == "arn:aws:iam::333333333333:saml-provider/ADFS"
    assert account_roles[0].role_arn == "arn:aws:iam::333333333333:role/ReadOnly"

def test_ask_which_role_to_assume_all_roles():
    account_names = {
        '333333333333': 'my-dev-account',
        '222222222222': 'my-prod-account'
    }
    principal_roles = [
        ['arn:aws:iam::333333333333:saml-provider/ADFS', 'arn:aws:iam::333333333333:role/ReadOnly'],
        ['arn:aws:iam::333333333333:saml-provider/ADFS', 'arn:aws:iam::333333333333:role/Admin'],
        ['arn:aws:iam::222222222222:saml-provider/ADFS', 'arn:aws:iam::222222222222:role/ReadOnly']
    ]
    account_name = 'all'
    role_name = "ReadOnly"

    account_roles = ask_which_role_to_assume(account_names, principal_roles, account_name, role_name)
    assert len(account_roles) == 2
    assert account_roles[0].account_name == 'my-dev-account'
    assert account_roles[0].role_name == "ReadOnly"
    assert account_roles[1].account_name == "my-prod-account"
    assert account_roles[1].role_name == "ReadOnly"


import boto3
import botocore
from botocore import client

from .consoleeffects import Colors


class AccountRole:

    def __init__(self, account_name, role_name, role_arn, principal_arn):
        self.account_name = account_name
        self.role_name = role_name
        self.role_arn = role_arn
        self.principal_arn = principal_arn


def ask_which_role_to_assume(account_names, principal_roles, account_name=None, role_name=None):
    roles_by_account = __get_roles_by_account(account_names, principal_roles)

    # User wants to assume a role on all accounts
    if account_name == 'all':
        return __get_all_account_roles(roles_by_account, role_name)
    # User wants to assume a role in a single account
    else:
        return __get_single_account_role(roles_by_account, account_name, role_name)


def assume_role(account_role, samlAssertion):
    conn = boto3.client('sts', config=client.Config(signature_version=botocore.UNSIGNED))
    aws_session_token = conn.assume_role_with_saml(
        RoleArn=account_role.role_arn,
        PrincipalArn=account_role.principal_arn,
        SAMLAssertion=samlAssertion,
        DurationSeconds=3600,
    )

    return aws_session_token


def __get_all_account_roles(roles_by_account, role_name):
    if not role_name:
        print("{}You must speicify a role using the {}--role{} option when assuming a role accross all accounts{}".format(Colors.lred,Colors.red,Colors.lred,Colors.normal))
        exit(1)
    account_roles = []
    for account_name in roles_by_account:
        if role_name in roles_by_account[account_name]:
            account_roles.append(roles_by_account[account_name][role_name])
        else:
            print("{}Will not try to assume role '{}{}{}' in account '{}{}{}' because you don't have access to that role{}".format(Colors.lyellow,Colors.cyan,role_name,Colors.lyellow,Colors.yellow,account_name,Colors.lyellow,Colors.normal))
    return account_roles


def __get_single_account_role(roles_by_account, account_name, role_name):
    # Prompt the user for account (if not already provided)
    if not account_name or not roles_by_account[account_name]:
        account_names = list(roles_by_account.keys())
        account_names.sort()
        account_name = __prompt_for_account(account_names)
    # Prompt the user for role (if not already provided)
    if not role_name or not roles_by_account[account_name][role_name]:
        role_names = __get_role_names_for_account(roles_by_account, account_name)
        role_names.sort()
        if len(role_names) == 1:
            role_name = role_names[0]
        else:
            role_name = __prompt_for_role(account_name, role_names)

    return [roles_by_account[account_name][role_name]]  # TODO - Return real list at some point


def __get_role_names_for_account(roles_by_account, account_name):
    role_names = []
    for role_name in roles_by_account[account_name]:
        role_names.append(role_name)
    return role_names

    
def __get_roles_by_account(account_names, principal_roles):
    roles_by_account = {}
    for (principal_arn, role_arn) in principal_roles:
        account_id = role_arn.split(':')[4]
        account_name = account_names[account_id]
        role_name = role_arn.split(':')[5].split('/')[1]

        roles_by_account.setdefault(account_name, {})[role_name] = AccountRole(account_name, role_name, role_arn, principal_arn)
    return roles_by_account


def __prompt_for_account(account_names):
    print('{}#--------------------------------------------------#'.format(Colors.lblue))
    print('#     {}To which account would you like to login?{}    #'.format(Colors.white,Colors.lblue))
    print('#--------------------------------------------------#{}'.format(Colors.normal))
    
    for index, account_name in enumerate(account_names):
        print("\t{}{}{}  {}{}".format(Colors.white,str(index).rjust(2), Colors.yellow,account_name,Colors.normal))
        
    while True:
        choice = input('{}Select account:{} '.format(Colors.lblue,Colors.normal))
        try:
            return account_names[int(choice)]
        except:
            maximum = len(account_names) - 1
            print('{}Please enter an integer between 0 and {}{}'.format(Colors.lred,maximum,Colors.normal))


def __prompt_for_role(account_name, role_names):
    border = ""
    spaces = ""
    for index in range(len(account_name)):
        border = "-" + border
        spaces = " " + spaces

    print('{}#------------------------------------------------{}#'.format(Colors.lblue,border))
    print('#   {}You have access to the following roles in {}{}{}   #'.format(Colors.white,Colors.yellow,account_name,Colors.lblue))
    print('#   {}Which role would you like to assume?{}{}         #'.format(Colors.white,Colors.lblue,spaces))
    print('#------------------------------------------------{}#{}'.format(border,Colors.normal))
    
    for index, role_name in enumerate(role_names):
        if role_name == "AccountAdministrator":
            print("\t{}{}  {}{}".format(Colors.red, str(index).rjust(2), role_name,Colors.normal))
        else:
            print("\t{}{}{}  {}{}".format(Colors.white, str(index).rjust(2), Colors.cyan, role_name, Colors.normal))
    
    while True:
        choice = input('{}Select role: {}'.format(Colors.lblue, Colors.normal))
        try:
            return role_names[int(choice)]
        except:
            maximum = len(role_names) - 1
            print('{}Please enter an integer between 0 and {}{}'.format(Colors.lred, maximum, Colors.normal))
    



from byu_awslogin.adfs_auth import authenticate
from unittest.mock import patch, MagicMock
import os
import json

@patch('byu_awslogin.adfs_auth.requests')
def test_authenticate(mock_requests):
    mock_session = mock_requests.Session.return_value = MagicMock()    
    
    # Mock initial login page
    script_dir = os.path.dirname(os.path.realpath(__file__))
    auth_page_text = open('{}/adfs_html_mocks/login_screen.html'.format(script_dir), 'r').read()
    auth_page_mock = MagicMock(status_code=200, text=auth_page_text)
    mock_session.get.side_effect = [auth_page_mock]
    
    # Mock duo required page
    duo_page_text = open('{}/adfs_html_mocks/duo_screen.html'.format(script_dir), 'r').read()
    duo_screen_mock = MagicMock(status_code=200, text=duo_page_text)
    
    # Mock duo init page
    duo_init_text = open('{}/adfs_html_mocks/duo_init.html'.format(script_dir), 'r').read()
    duo_init_url = "https://REDACTED.duosecurity.com/frame/prompt?sid=REDACTED"
    duo_init_mock = MagicMock(status_code=200, text=duo_init_text, url=duo_init_url)
    
    # Mock duo init auth transaction
    duo_auth_trans_init_json = open('{}/adfs_html_mocks/duo-trans-init.json'.format(script_dir), 'r').read()
    duo_auth_trans_init_mock = MagicMock(status_code=200, text=duo_auth_trans_init_json)
    duo_auth_trans_init_mock.json.return_value = json.loads(duo_auth_trans_init_json)
    
    # Mock duo verify code
    duo_verify_code_json = open('{}/adfs_html_mocks/duo_trans_verify.json'.format(script_dir), 'r').read()
    duo_verify_code_mock = MagicMock(status_code=200, text=duo_verify_code_json)
    duo_verify_code_mock.json.return_value = json.loads(duo_verify_code_json)
    
    # Mock duo auth result
    duo_auth_result_json = open('{}/adfs_html_mocks/duo_auth_result.json'.format(script_dir), 'r').read()
    duo_auth_result_mock = MagicMock(status_code=200, text=duo_auth_result_json)
    duo_auth_result_mock.json.return_value = json.loads(duo_auth_result_json)
    
    mock_session.post.side_effect = [duo_screen_mock, duo_init_mock, duo_auth_trans_init_mock, duo_verify_code_mock, duo_auth_result_mock]
    login_response_html_soup, session, auth_signature, duo_request_signature = authenticate('FakeUsername', 'FakePassword')
    assert auth_signature == 'REDACTED'
    assert duo_request_signature == 'TX|REDACTED:APP|REDACTED'